let

type Priority-Queue a =
  class
    method insert : a -> ()
    method delete : a
  end

exception Delete

function ordlist <a> (ord : (a, a) -> Bool) : Priority-Queue =
  class
    local
      val queue = ref Nil
    in
      method insert (elem : a) : () =
        let
          function ins (list : List a) : List a =
            case list of Nil       => Cons ( head = elem, tail = Nil )
                      |  Cons cons => if ord (elem, cons.head) 
                                      then Cons ( head = elem, tail = list )
                                      else Cons ( head = cons.head, tail = ins (cons.tail) )
            end
        in
          queue := ins (!queue)
        end
      method delete : a =
        case !queue of Nil       => throw Delete
                    |  Cons cons => queue := cons.tail; cons.head
        end
    end
  end

val q = new (ordlist <Nat> (\ (m, n) => m > n ))

in
q.insert 815;
q.insert 4711;
q.insert 0;
System.IO.println (q.delete);
System.IO.println (q.delete);
System.IO.println (q.delete)
end